@@ -255,12 +255,12 @@ impl<'a, I: Tokens> Parser<I> {
                             self.emit_err(span, SyntaxError::ImportMetaInScript);
                         }
                         return self
-                            .parse_import_meta_prop(import)
+                            .parse_import_meta_prop(Import { span: import.span })
                             .map(Expr::MetaProp)
                             .map(Box::new);
                     }
 
-                    return self.parse_dynamic_import(start, import);
+                    return self.parse_dynamic_import(start, import.span);
                 }
 
                 tok!("async") => {
@@ -479,18 +479,19 @@ impl<'a, I: Tokens> Parser<I> {
     }
 
     /// `parseImportMetaProperty`
-    pub(super) fn parse_import_meta_prop(&mut self, import: Ident) -> PResult<MetaPropExpr> {
-        let meta = import;
-
+    pub(super) fn parse_import_meta_prop(&mut self, import: Import) -> PResult<MetaPropExpr> {
         expect!(self, '.');
 
-        let prop = if is!(self, "meta") {
+        let _ = if is!(self, "meta") {
             self.parse_ident_name()?
         } else {
             unexpected!(self, "meta");
         };
 
-        Ok(MetaPropExpr { meta, prop })
+        Ok(MetaPropExpr {
+            span: span!(self, import.span.lo()),
+            kind: MetaPropKind::ImportMeta,
+        })
     }
 
     /// `is_new_expr`: true iff we are parsing production 'NewExpression'.
@@ -499,16 +500,14 @@ impl<'a, I: Tokens> Parser<I> {
 
         let start = cur_pos!(self);
         if eat!(self, "new") {
-            let span_of_new = span!(self, start);
             if eat!(self, '.') {
-                let start_of_target = cur_pos!(self);
                 if eat!(self, "target") {
                     let expr = Box::new(Expr::MetaProp(MetaPropExpr {
-                        meta: Ident::new(js_word!("new"), span_of_new),
-                        prop: Ident::new(js_word!("target"), span!(self, start_of_target)),
+                        span: span!(self, start),
+                        kind: MetaPropKind::NewTarget,
                     }));
 
-                    return self.parse_subscripts(ExprOrSuper::Expr(expr), true);
+                    return self.parse_subscripts(Callee::Expr(expr), true);
                 }
 
                 unexpected!(self, "target")
@@ -535,7 +534,7 @@ impl<'a, I: Tokens> Parser<I> {
                 // Parsed with 'MemberExpression' production.
                 let args = self.parse_args(false).map(Some)?;
 
-                let new_expr = ExprOrSuper::Expr(Box::new(Expr::New(NewExpr {
+                let new_expr = Callee::Expr(Box::new(Expr::New(NewExpr {
                     span: span!(self, start),
                     callee,
                     args,
@@ -558,7 +557,13 @@ impl<'a, I: Tokens> Parser<I> {
         }
 
         if eat!(self, "super") {
-            let base = ExprOrSuper::Super(Super {
+            let base = Callee::Super(Super {
+                span: span!(self, start),
+            });
+            return self.parse_subscripts(base, true);
+        }
+        if eat!(self, "import") {
+            let base = Callee::Import(Import {
                 span: span!(self, start),
             });
             return self.parse_subscripts(base, true);
@@ -566,7 +571,7 @@ impl<'a, I: Tokens> Parser<I> {
         let obj = self.parse_primary_expr()?;
         return_if_arrow!(self, obj);
 
-        self.parse_subscripts(ExprOrSuper::Expr(obj), true)
+        self.parse_subscripts(Callee::Expr(obj), true)
     }
 
     /// Parse `NewExpression`.
@@ -779,7 +784,7 @@ impl<'a, I: Tokens> Parser<I> {
             // It's a call expression
             return Ok(Box::new(Expr::Call(CallExpr {
                 span: span!(self, async_span.lo()),
-                callee: ExprOrSuper::Expr(Box::new(Expr::Ident(Ident::new(
+                callee: Callee::Expr(Box::new(Expr::Ident(Ident::new(
                     "async".into(),
                     async_span,
                 )))),
@@ -965,12 +970,12 @@ impl<'a, I: Tokens> Parser<I> {
         })
     }
 
-    fn parse_subscripts(&mut self, mut obj: ExprOrSuper, no_call: bool) -> PResult<Box<Expr>> {
+    fn parse_subscripts(&mut self, mut obj: Callee, no_call: bool) -> PResult<Box<Expr>> {
         let start = obj.span().lo;
         loop {
             obj = match self.parse_subscript(start, obj, no_call)? {
                 (expr, false) => return Ok(expr),
-                (expr, true) => ExprOrSuper::Expr(expr),
+                (expr, true) => Callee::Expr(expr),
             }
         }
     }
@@ -979,7 +984,7 @@ impl<'a, I: Tokens> Parser<I> {
     fn parse_subscript(
         &mut self,
         start: BytePos,
-        mut obj: ExprOrSuper,
+        mut obj: Callee,
         no_call: bool,
     ) -> PResult<(Box<Expr>, bool)> {
         let _ = cur!(self, false);
@@ -990,8 +995,21 @@ impl<'a, I: Tokens> Parser<I> {
                 assert_and_bump!(self, '!');
 
                 let expr = match obj {
-                    ExprOrSuper::Super(..) => unimplemented!("super!"),
-                    ExprOrSuper::Expr(expr) => expr,
+                    Callee::Super(..) => {
+                        syntax_error!(
+                            self,
+                            self.input.cur_span(),
+                            SyntaxError::TsNonNullAssertionNotAllowed(js_word!("super"))
+                        )
+                    }
+                    Callee::Import(..) => {
+                        syntax_error!(
+                            self,
+                            self.input.cur_span(),
+                            SyntaxError::TsNonNullAssertionNotAllowed(js_word!("import"))
+                        )
+                    }
+                    Callee::Expr(expr) => expr,
                 };
                 return Ok((
                     Box::new(Expr::TsNonNull(TsNonNullExpr {
@@ -1004,13 +1022,13 @@ impl<'a, I: Tokens> Parser<I> {
 
             if {
                 match obj {
-                    ExprOrSuper::Expr(..) => true,
-                    // super() cannot be generic
+                    Callee::Expr(..) => true,
+                    // super() or import() cannot be generic
                     _ => false,
                 }
             } && is!(self, '<')
             {
-                let is_dynamic_import = is_import(&obj);
+                let is_dynamic_import = obj.is_import();
 
                 let mut obj_opt = Some(obj);
                 // tsTryParseAndCatch is expensive, so avoid if not necessary.
@@ -1023,7 +1041,7 @@ impl<'a, I: Tokens> Parser<I> {
                 let result = self.try_parse_ts(|p| {
                     if !no_call
                         && p.at_possible_async(match &mut_obj_opt {
-                            Some(ExprOrSuper::Expr(ref expr)) => &**expr,
+                            Some(Callee::Expr(ref expr)) => &**expr,
                             _ => unreachable!(),
                         })?
                     {
@@ -1054,7 +1072,7 @@ impl<'a, I: Tokens> Parser<I> {
                     } else if is!(p, '`') {
                         p.parse_tagged_tpl(
                             match mut_obj_opt {
-                                Some(ExprOrSuper::Expr(obj)) => obj.take(),
+                                Some(Callee::Expr(obj)) => obj.take(),
                                 _ => unreachable!(),
                             },
                             Some(type_args),
@@ -1113,14 +1131,46 @@ impl<'a, I: Tokens> Parser<I> {
             expect!(self, ']');
             let span = Span::new(obj.span().lo(), self.input.last_pos(), Default::default());
             debug_assert_eq!(obj.span().lo(), span.lo());
+            let prop = ComputedPropName { span, expr: prop };
 
             return Ok((
-                Box::new(wrap!(Expr::Member(MemberExpr {
-                    span,
-                    obj,
-                    prop,
-                    computed: true,
-                }))),
+                Box::new(match obj {
+                    Callee::Import(..) => unreachable!(),
+                    Callee::Super(obj) => {
+                        if let Some(_) = question_dot_token {
+                            if no_call {
+                                syntax_error!(
+                                    self,
+                                    self.input.cur_span(),
+                                    SyntaxError::InvalidSuperCall
+                                );
+                            }
+                            syntax_error!(self, self.input.cur_span(), SyntaxError::InvalidSuper);
+                        } else {
+                            Expr::SuperProp(SuperPropExpr {
+                                span,
+                                obj,
+                                prop: SuperProp::Computed(prop),
+                            })
+                        }
+                    }
+                    Callee::Expr(obj) => {
+                        let expr = Expr::Member(MemberExpr {
+                            span,
+                            obj,
+                            prop: MemberProp::Computed(prop),
+                        });
+                        if let Some(question_dot_token) = question_dot_token {
+                            Expr::OptChain(OptChainExpr {
+                                span: span!(self, start),
+                                question_dot_token,
+                                expr: Box::new(expr),
+                            })
+                        } else {
+                            expr
+                        }
+                    }
+                }),
                 true,
             ));
         }
@@ -1131,7 +1181,7 @@ impl<'a, I: Tokens> Parser<I> {
             && eat!(self, '.'))
             || (!no_call && (is!(self, '(')))
         {
-            let args = self.parse_args(is_import(&obj))?;
+            let args = self.parse_args(obj.is_import())?;
             return Ok((
                 Box::new(wrap!(Expr::Call(CallExpr {
                     span: span!(self, start),
@@ -1146,28 +1196,79 @@ impl<'a, I: Tokens> Parser<I> {
         // member expression
         // $obj.name
         if eat!(self, '.') {
-            let prop: Box<Expr> = Box::new(self.parse_maybe_private_name().map(|e| match e {
-                Either::Left(p) => Expr::PrivateName(p),
-                Either::Right(i) => Expr::Ident(i),
-            })?);
+            let prop = self.parse_maybe_private_name().map(|e| match e {
+                Either::Left(p) => MemberProp::PrivateName(p),
+                Either::Right(i) => MemberProp::Ident(i),
+            })?;
             let span = span!(self, obj.span().lo());
             debug_assert_eq!(obj.span().lo(), span.lo());
             debug_assert_eq!(prop.span().hi(), span.hi());
 
             return Ok((
-                Box::new(wrap!(Expr::Member(MemberExpr {
-                    span,
-                    obj,
-
-                    prop,
-                    computed: false,
-                }))),
+                Box::new(match obj {
+                    Callee::Import(_) => {
+                        if let MemberProp::Ident(Ident {
+                            sym: js_word!("meta"),
+                            ..
+                        }) = prop
+                        {
+                            if !self.ctx().can_be_module {
+                                let span = span!(self, start);
+                                self.emit_err(span, SyntaxError::ImportMetaInScript);
+                            }
+                            Expr::MetaProp(MetaPropExpr {
+                                span,
+                                kind: MetaPropKind::ImportMeta,
+                            })
+                        } else {
+                            unexpected!(self, "meta");
+                        }
+                    }
+                    Callee::Super(obj) => {
+                        if let Some(_) = question_dot_token {
+                            if no_call {
+                                syntax_error!(
+                                    self,
+                                    self.input.cur_span(),
+                                    SyntaxError::InvalidSuperCall
+                                );
+                            }
+                            syntax_error!(self, self.input.cur_span(), SyntaxError::InvalidSuper);
+                        } else {
+                            match prop {
+                                MemberProp::Ident(ident) => Expr::SuperProp(SuperPropExpr {
+                                    span,
+                                    obj,
+                                    prop: SuperProp::Ident(ident),
+                                }),
+                                MemberProp::PrivateName(..) => syntax_error!(
+                                    self,
+                                    self.input.cur_span(),
+                                    SyntaxError::InvalidSuperCall
+                                ),
+                                MemberProp::Computed(..) => unreachable!(),
+                            }
+                        }
+                    }
+                    Callee::Expr(obj) => {
+                        let expr = Expr::Member(MemberExpr { span, obj, prop });
+                        if let Some(question_dot_token) = question_dot_token {
+                            Expr::OptChain(OptChainExpr {
+                                span: span!(self, start),
+                                question_dot_token,
+                                expr: Box::new(expr),
+                            })
+                        } else {
+                            expr
+                        }
+                    }
+                }),
                 true,
             ));
         }
 
         match obj {
-            ExprOrSuper::Expr(expr) => {
+            Callee::Expr(expr) => {
                 // MemberExpression[?Yield, ?Await] TemplateLiteral[?Yield, ?Await, +Tagged]
                 if is!(self, '`') {
                     let tpl = self.parse_tagged_tpl(expr, None)?;
@@ -1176,12 +1277,15 @@ impl<'a, I: Tokens> Parser<I> {
 
                 Ok((expr, false))
             }
-            ExprOrSuper::Super(..) => {
+            Callee::Super(..) => {
                 if no_call {
                     syntax_error!(self, self.input.cur_span(), SyntaxError::InvalidSuperCall);
                 }
                 syntax_error!(self, self.input.cur_span(), SyntaxError::InvalidSuper);
             }
+            Callee::Import(..) => {
+                syntax_error!(self, self.input.cur_span(), SyntaxError::InvalidImport);
+            }
         }
     }
     /// Parse call, dot, and `[]`-subscript expressions.
@@ -1225,7 +1329,13 @@ impl<'a, I: Tokens> Parser<I> {
 
         // `super()` can't be handled from parse_new_expr()
         if eat!(self, "super") {
-            let obj = ExprOrSuper::Super(Super {
+            let obj = Callee::Super(Super {
+                span: span!(self, start),
+            });
+            return self.parse_subscripts(obj, false);
+        }
+        if eat!(self, "import") {
+            let obj = Callee::Import(Import {
                 span: span!(self, start),
             });
             return self.parse_subscripts(obj, false);
@@ -1267,8 +1377,15 @@ impl<'a, I: Tokens> Parser<I> {
         if is!(self, '(') {
             // This is parsed using production MemberExpression,
             // which is left-recursive.
-            let callee = ExprOrSuper::Expr(callee);
-            let args = self.parse_args(is_import(&callee))?;
+            let (callee, is_import) = match &*callee {
+                Expr::Ident(Ident {
+                    sym: js_word!("import"),
+                    span,
+                    ..
+                }) => (Callee::Import(Import { span: *span }), true),
+                _ => (Callee::Expr(callee), false),
+            };
+            let args = self.parse_args(is_import)?;
 
             let call_expr = Box::new(Expr::Call(CallExpr {
                 span: span!(self, start),
@@ -1278,7 +1395,7 @@ impl<'a, I: Tokens> Parser<I> {
                 type_args,
             }));
 
-            return self.parse_subscripts(ExprOrSuper::Expr(call_expr), false);
+            return self.parse_subscripts(Callee::Expr(call_expr), false);
         }
         if type_args.is_some() {
             // This fails
@@ -1667,17 +1784,17 @@ impl<'a, I: Tokens> Parser<I> {
     pub(super) fn parse_dynamic_import(
         &mut self,
         start: BytePos,
-        import_ident: Ident,
+        import_span: Span,
     ) -> PResult<Box<Expr>> {
         let args = self.parse_args(true)?;
         let import = Box::new(Expr::Call(CallExpr {
             span: span!(self, start),
-            callee: ExprOrSuper::Expr(Box::new(Expr::Ident(import_ident))),
+            callee: Callee::Import(Import { span: import_span }),
             args,
             type_args: Default::default(),
         }));
 
-        self.parse_subscripts(ExprOrSuper::Expr(import), true)
+        self.parse_subscripts(Callee::Expr(import), true)
     }
 
     pub(super) fn check_assign_target(&mut self, expr: &Expr, deny_call: bool) {
@@ -1719,16 +1836,3 @@ impl<'a, I: Tokens> Parser<I> {
         }
     }
 }
-
-fn is_import(obj: &ExprOrSuper) -> bool {
-    match *obj {
-        ExprOrSuper::Expr(ref expr) => match **expr {
-            Expr::Ident(Ident {
-                sym: js_word!("import"),
-                ..
-            }) => true,
-            _ => false,
-        },
-        _ => false,
-    }
-}
